Package: tinysnapshot
Type: Package
Title: Snapshots for Unit Tests using the 'tinytest' Framework
Version: 0.3.0
Authors@R: 
    c(person(given = "Vincent",
             family = "Arel-Bundock",
             role = c("aut", "cre"),
             email = "vincent.arel-bundock@umontreal.ca",
             comment = c(ORCID = "0000-0003-2042-7063")))
Maintainer: Vincent Arel-Bundock <vincent.arel-bundock@umontreal.ca>
Description: Snapshots for unit tests using the 'tinytest' framework for R. Includes expectations to test base R and 'ggplot2' plots as well as console output from print().
License: GPL (>= 3)
URL: https://github.com/vincentarelbundock/tinysnapshot
BugReports: https://github.com/vincentarelbundock/tinysnapshot/issues
Imports: diffobj, magick (>= 2.7.4), tinytest (>= 1.4.1)
Suggests: fontquiver, ggplot2, ragg, rsvg, svglite
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-20 12:22:21 UTC; vincent
Author: Vincent Arel-Bundock [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2042-7063>)
Repository: CRAN
Date/Publication: 2026-08-21 06:50:02 UTC
Built: R 4.5.2; ; 2026-08-21 08:59:39 UTC; unix
